<?php
/* --------------------------------------------------------------
   OrderListGenerator.inc.php 2016-04-15
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2016 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

MainFactory::load_class('OrderListGeneratorInterface');

/**
 * Class OrderListGenerator
 *
 * @category System
 * @package  Order
 */
class OrderListGenerator implements OrderListGeneratorInterface
{
	/**
	 * @var CI_DB_query_builder
	 */
	protected $db;
	
	/**
	 * @var int
	 */
	protected $defaultLanguageId; 
	
	/**
	 * OrderListGenerator Constructor
	 *
	 * @param \CI_DB_query_builder $db
	 */
	public function __construct(CI_DB_query_builder $db)
	{
		$this->db = $db;
		
		$query = 'SELECT languages_id FROM languages WHERE code = "' . DEFAULT_LANGUAGE . '"'; 
		$this->defaultLanguageId = $this->db->query($query)->row()->languages_id; 
	}
	
	
	/**
	 * Get Order List Items
	 *
	 * Returns an order list item collection.
	 *
	 * @param array       $conditions Associative array which holds the CI_DB_query_builder conditions (default empty
	 *                                array).
	 * @param \IntType    $startIndex The start index of the wanted array to be returned (default = null).
	 * @param \IntType    $maxCount   Maximum amount of items which should be returned (default = null).
	 * @param \StringType $orderBy    A string which defines how the items should be ordered (default = null).
	 *
	 * @return \OrderListItemCollection
	 * 
	 * @throws InvalidArgumentException If the result rows contain invalid values.
	 */
	public function getOrderListByConditions(array $conditions = array(),
	                                         IntType $startIndex = null,
	                                         IntType $maxCount = null,
	                                         StringType $orderBy = null)
	{
		$this->_select()->_limit($startIndex, $maxCount)->_order($orderBy)->_group();
		
		$this->db->where('orders_status.language_id', $this->defaultLanguageId)
			->where('orders_total.class', 'ot_total'); 
		
		if(count($conditions) > 0)
		{
			$this->db->where($conditions);
		}
		
		$result = $this->db->get()->result_array();
		
		return $this->_prepareCollection($result);
	}
	
	
	/**
	 * Filter records by a single keyword string.
	 *
	 * @param \StringType      $keyword    Keyword string to be used for searching in order records.
	 * @param \IntType|null    $startIndex The start index of the wanted array to be returned (default = null).
	 * @param \IntType|null    $maxCount   Maximum amount of items which should be returned (default = null).
	 * @param \StringType|null $orderBy    A string which defines how the items should be ordered (default = null).
	 *
	 * @return mixed
	 * 
	 * @throws InvalidArgumentException If the result rows contain invalid values.
 	 */
	public function getOrderListByKeyword(StringType $keyword,
	                                      IntType $startIndex = null,
	                                      IntType $maxCount = null,
	                                      StringType $orderBy = null)
	{
		$this->_select()->_limit($startIndex, $maxCount)->_order($orderBy)->_group();
		
		$match = $this->db->escape_like_str($keyword->asString());
		
		$this->db->where('
			orders_total.class = "ot_total" 
			AND orders_status.language_id = ' . $this->defaultLanguageId . ' 
			AND ( 
				orders.orders_id LIKE "%' . $match . '%"
				OR orders.customers_id LIKE "%' . $match . '%"
				OR orders.date_purchased LIKE "%' . $match . '%"
				OR orders.payment_class LIKE "%' . $match . '%"
				OR orders.payment_method LIKE "%' . $match . '%"
				OR orders.shipping_class LIKE "%' . $match . '%"
				OR orders.shipping_method LIKE "%' . $match . '%"
				OR orders.customers_firstname LIKE "%' . $match . '%"
				OR orders.customers_lastname LIKE "%' . $match . '%"
				OR orders_total.value LIKE "%' . $match . '%"
				OR orders_status.orders_status_id LIKE "%' . $match . '%"
				OR orders_status.orders_status_name LIKE "%' . $match . '%"
			)');
		
		$result = $this->db->get()->result_array();
		
		return $this->_prepareCollection($result);
	}
	
	
	/**
	 * Execute the select and join methods.
	 *
	 * @return OrderListGenerator Returns the instance object for method chaining.
	 */
	protected function _select()
	{
		$this->db->select('orders.orders_id, orders.customers_id, orders.date_purchased, orders.payment_class,
			orders.payment_method, orders.shipping_class, orders.shipping_method, orders.customers_firstname, 
			orders.customers_lastname, orders_status.orders_status_id, orders_status.orders_status_name, 
			orders_total.value AS total_sum')
		         ->from('orders')
		         ->join('orders_status', 'orders_status.orders_status_id = orders.orders_status', 'inner')
		         ->join('orders_total', 'orders_total.orders_id = orders.orders_id', 'left');
		
		return $this;
	}
	
	
	/**
	 * Add limit configuration to the database object.
	 *
	 * @param \IntType $startIndex
	 * @param \IntType $maxCount
	 *
	 * @return OrderListGenerator Returns the instance object for method chaining.
	 */
	protected function _limit(IntType $startIndex = null, IntType $maxCount = null)
	{
		if($maxCount && $startIndex)
		{
			$this->db->limit($maxCount->asInt(), $startIndex->asInt());
		}
		else
		{
			if($maxCount && !$startIndex)
			{
				$this->db->limit($maxCount->asInt());
			}
		}
		
		return $this;
	}
	
	
	/**
	 * Set the order by clause of the query.
	 *
	 * @param \StringType $orderBy
	 *
	 * @return OrderListGenerator Returns the instance object for method chaining.
	 */
	protected function _order(StringType $orderBy = null)
	{
		if($orderBy)
		{
			$this->db->order_by($orderBy->asString());
		}
		
		return $this;
	}
	
	
	/**
	 * Execute the group by statement.
	 *
	 * @return OrderListGenerator Returns the instance object for method chaining.
	 */
	protected function _group()
	{
		$this->db->group_by('orders.orders_id, orders_status.orders_status_name, orders_total.value');
		
		return $this;
	}
	
	
	/**
	 * Prepare the OrderListItemCollection object.
	 *
	 * This method will prepare the collection object which is going to be returned by both
	 * the "get" and "filter" methods. The following values are required to be present in
	 * each row of the $result parameter:
	 *      - orders_id
	 *      - customers_id
	 *      - customers_firstname
	 *      - customers_lastname
	 *      - date_purchased
	 *      - payment_class
	 *      - payment_method
	 *      - shipping_class
	 *      - shipping_method
	 *      - orders_status_id
	 *      - orders_status_name
	 *      - total_sum
	 *
	 * @param array $result Contains the order data.
	 *
	 * @return \OrderListItemCollection
	 * 
	 * @throws InvalidArgumentException
	 */
	protected function _prepareCollection(array $result)
	{
		$listItems = array();
		
		foreach($result as $row)
		{
			$orderId          = new IdType((int)$row['orders_id']);
			$customerId       = new IdType((int)$row['customers_id']);
			$customerName     = new StringType($row['customers_firstname'] . ' ' . $row['customers_lastname']);
			$totalSum         = new DecimalType((float)$row['total_sum']);
			$paymentType      = MainFactory::create('OrderPaymentType', new StringType($row['payment_method']),
			                                        new StringType($row['payment_class']));
			$shippingType     = MainFactory::create('OrderShippingType', new StringType($row['shipping_method']),
			                                        new StringType($row['shipping_class']));
			$purchaseDateTime = new EmptyDateTime($row['date_purchased']);
			$orderStatusId    = new IdType((int)$row['orders_status_id']);
			$orderStatusName  = new StringType($row['orders_status_name']);
			
			$orderListItem = MainFactory::create('OrderListItem', $orderId, $customerId, $customerName, $totalSum,
			                                     $paymentType, $shippingType, $purchaseDateTime, $orderStatusId,
			                                     $orderStatusName);
			$listItems[]   = $orderListItem;
		}
		
		$collection = MainFactory::create('OrderListItemCollection', $listItems);
		
		return $collection;
	}
}